import * as React from 'react';
import { useTheme } from '@mui/material/styles';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import Container from '@mui/material/Container';
import Fade from '@mui/material/Fade';
import Paper, { PaperProps } from '@mui/material/Paper';
import Typography from '@mui/material/Typography';
import Tabs from '@mui/material/Tabs';
import Tab from '@mui/material/Tab';
import KeyboardArrowRightRounded from '@mui/icons-material/KeyboardArrowRightRounded';
import Link from 'docs/src/modules/components/Link';
import PricingTable, { PlanName, PlanPrice } from 'docs/src/components/pricing/PricingTable';

const Plan = React.forwardRef<
  HTMLDivElement,
  {
    plan: 'community' | 'pro' | 'premium';
    benefits?: Array<string>;
    unavailable?: boolean;
  } & PaperProps
>(function Plan({ plan, benefits, unavailable, sx, ...props }, ref) {
  const globalTheme = useTheme();
  const mode = globalTheme.palette.mode;
  return (
    <Paper
      ref={ref}
      variant="outlined"
      sx={{ p: 2, ...(unavailable && { '& .MuiTypography-root': { opacity: 0.5 } }), ...sx }}
      {...props}
    >
      <PlanName plan={plan} />
      <Box {...(plan === 'community' && { my: 2 })} {...(plan === 'premium' && { mb: 2 })}>
        <PlanPrice plan={plan} />
      </Box>
      {unavailable ? (
        <Button
          variant="outlined"
          disabled
          fullWidth
          sx={{ py: 1, '&.Mui-disabled': { color: 'text.disabled' } }}
        >
          In progress!
        </Button>
      ) : (
        <Button
          variant={plan.match(/(pro|premium)/) ? 'contained' : 'outlined'}
          fullWidth
          component={Link}
          noLinkStyle
          href={
            {
              community: '/material-ui/getting-started/usage/',
              pro: 'https://mui.com/store/items/mui-x-pro/',
              premium: 'https://mui.com/store/items/mui-x-premium/',
            }[plan]
          }
          endIcon={<KeyboardArrowRightRounded />}
          sx={{ py: 1 }}
        >
          {plan.match(/(pro|premium)/) ? 'Buy now' : 'Get started'}
        </Button>
      )}
      {benefits &&
        benefits.map((text) => (
          <Box key={text} sx={{ display: 'flex', alignItems: 'center', mt: 2 }}>
            <img src={`/static/branding/pricing/yes-${mode}.svg`} alt="" />
            <Typography
              variant="body2"
              color="text.secondary"
              fontWeight="extraBold"
              sx={{ ml: 1 }}
            >
              {text}
            </Typography>
          </Box>
        ))}
    </Paper>
  );
});

export default function PricingList() {
  const [planIndex, setPlanIndex] = React.useState(0);
  return (
    <Container sx={{ pb: 2, mt: '-1px', display: { xs: 'block', md: 'none' } }}>
      <Tabs
        value={planIndex}
        variant="fullWidth"
        onChange={(event, value) => setPlanIndex(value)}
        sx={{
          mb: 2,
          position: 'sticky',
          top: 55,
          bgcolor: 'background.paper',
          zIndex: 1,
          mx: { xs: -2, sm: -3 },
          borderTop: '1px solid',
          borderColor: 'divider',
          '& .MuiTab-root': {
            borderBottom: '1px solid',
            borderColor: 'divider',
            '&.Mui-selected': {
              bgcolor: (theme) => (theme.palette.mode === 'dark' ? 'primaryDark.700' : 'grey.50'),
            },
          },
        }}
      >
        <Tab label="Community" />
        <Tab
          label="Pro"
          sx={{ borderWidth: '0 1px 0 1px', borderStyle: 'solid', borderColor: 'divider' }}
        />
        <Tab label="Premium" />
      </Tabs>
      {planIndex === 0 && (
        <Fade in>
          <div>
            <Plan plan="community" />
            <PricingTable columnHeaderHidden plans={['community']} />
          </div>
        </Fade>
      )}
      {planIndex === 1 && (
        <Fade in>
          <div>
            <Plan plan="pro" />
            <PricingTable columnHeaderHidden plans={['pro']} />
          </div>
        </Fade>
      )}
      {planIndex === 2 && (
        <Fade in>
          <div>
            <Plan plan="premium" />
            <PricingTable columnHeaderHidden plans={['premium']} />
          </div>
        </Fade>
      )}
    </Container>
  );
}
